package PureMessage::Install::Full;
use strict;
use PureMessage::Install;
use PureMessage::Install::Custom;
use Sophos::IP::Iface qw(interfaces);
our @ISA = ("PureMessage::Install");

sub new {
    my $class = shift;
    my %opts = @_;
    
    my $self = PureMessage::Install::Custom->new( roles => [ qw(edge csm mta
                                                                database euui historiandb)
                                                           ],
                                                  %opts );

    # Don't prompt for the MTA question
    $self->{role_opts}{mta}{must_prompt} = 0;
    $self->set_instopt("mta", "Postfix");

    # Add our IP to PostgreSQL trusted hosts (default is the whole subnet)
    my @ifs = interfaces;
    if (@ifs) {
        my $ext_ips = join " ", map {$_->{ip}} @ifs;
        $self->prompt_user("pg_trusted_hosts", 0);
        $self->set_instopt("pg_trusted_hosts", $ext_ips);
    }

    return bless $self, $class;
}

1;

__END__

=head1 NAME

PureMessage::Install::Full

=head1 SYNOPSIS

  use PureMessage::Install::Full;
  my $inst = PureMessage::Install::Full->new();

=head1 DESCRIPTION

Perform a full installation that installs a complete, working PureMessage
system.  Only the necessary questions should be asked.  The focus here is to
have reasonable, working defaults.

=head1 CONFIGURATION

=head1 AUTHOR

Sophos PLC

=head1 COPYRIGHT

Copyright (C) 2004-2017 Sophos PLC
All Rights Reserved.

=head1 SEE ALSO

pmx-setup-v6


