#!/usr/bin/perl
# PureMessage Installer - Shell Manipulation

package PureMessage::Install::Shell;

use strict;
use warnings;
use Exporter;
use base qw(Exporter);

use vars qw(%Shells @EXPORT);
sub write_all_shell_files ($$$$);

@EXPORT = qw(write_all_shell_files);

%Shells = (
     sh => {
         name           => q{.pmx_sh_vars},
         parents        => [ qw{.bash_profile .profile} ],
         source         => q{.},
         contents       => <<'!SH!',
     
# PureMessage Bourne Shell Environment
# This file was automatically generated by PureMessage-Install [[version]].
#

if test -z "${PATH:-}"; then
    PATH="/usr/bin:/bin"
fi

PATH="[[prefix]]/bin:[[prefix]]/etc/init.d:$PATH"

if test -z "${[[ldpath]]:-}"; then
    [[ldpath]]="[[prefix]]/lib"
else
    [[ldpath]]="[[prefix]]/lib:$[[ldpath]]"
fi

if test -z "${LD_32_LIBRARY_PATH:-}"; then
    LD_32_LIBRARY_PATH="[[prefix]]/lib"
else
    LD_32_LIBRARY_PATH="[[prefix]]/lib:$LD_32_LIBRARY_PATH"
fi

if test -z "${MANPATH:-}"; then
    MANPATH="[[extra_man]]"
else
    MANPATH="[[extra_man]]:$MANPATH"
fi

OPENSSL_CONF="[[prefix]]/etc/manager/openssl.cnf"

export MANPATH PATH [[ldpath]] LD_32_LIBRARY_PATH OPENSSL_CONF

# set http_proxy from install time if set
install_time_http_proxy="[[http_proxy]]"
if test -n "$install_time_http_proxy" && test -z "${http_proxy:-}" && test -z "${HTTP_proxy:-}"; then
    http_proxy="$install_time_http_proxy"
    export http_proxy
fi
unset install_time_http_proxy

# EOF
!SH!
     },
     
     csh => {
         name            => q{.pmx_csh_vars},
         parents         => [ qw{.login} ],
         source          => q{source},
         contents        => <<'!CSH!',
         
# PureMessage C Shell Environment
# This file was automatically generated by PureMessage-Install [[version]].
#
         
setenv PATH "[[prefix]]/bin:[[prefix]]/etc/init.d:"$PATH

if ($?[[ldpath]]) then
    setenv [[ldpath]] "[[prefix]]/lib:"$[[ldpath]]
else
    setenv [[ldpath]] "[[prefix]]/lib"
endif

if ($?MANPATH) then
    setenv MANPATH "[[extra_man]]:"$MANPATH
else
    setenv MANPATH "[[extra_man]]"
endif

if ($?LD_32_LIBRARY_PATH) then
    setenv LD_32_LIBRARY_PATH "[[prefix]]/lib:"$LD_32_LIBRARY_PATH
else
    setenv LD_32_LIBRARY_PATH "[[prefix]]/lib"
endif

# set http_proxy from install time if set
if ( ("[[http_proxy]]" != "" ) && !($?http_proxy) && !($?HTTP_proxy) ) then
     setenv http_proxy "[[http_proxy]]"
endif

setenv OPENSSL_CONF "[[prefix]]/etc/manager/openssl.cnf"

# EOF
!CSH!
     },
);

     
return 1;


sub write_env_file ($$$$$) {
    my($file, $uid, $gid, $contents, $subst) = @_;
    
    $contents =~ s{\[\[(.+?)\]\]}{defined($subst->{$1}) ? $subst->{$1} : ""}eg;
    
    if(open(ENV_FILE, ">$file")) {
        print ENV_FILE $contents;
        close(ENV_FILE);
        if(chown($uid, $gid, $file)) {
            return 1;
        } else {
            warn "Changing permissions of $file failed: $!";
            return;
        }
    } else {
        warn "Writing to $file failed: $!";
        return;
    }
}


sub write_source_file ($$$$) {
    my($file, $uid, $gid, $source_line) = @_;
    my($mode, $newmode);
    
    if(-e($file)) {
        $mode = ((stat($file))[2]) & 07777;
        $newmode = $mode | 0200; # +w

        if($newmode != $mode) {
            unless(chmod($newmode, $file)) {
                warn "Make writable on $file failed: $!";
                return;
            }
        }
        
        if(open(SOURCE_FILE, $file)) {
            if(grep(substr(
                $_, 0, length($source_line)) eq $source_line, <SOURCE_FILE>
            )) {
                # already got it
                close(SOURCE_FILE);
                chmod($mode, $file) if($newmode != $mode);
                return 1;
            }
            close(SOURCE_FILE);
        } else {
            warn "Reading $file failed: $!";
            chmod($mode, $file) if($newmode != $mode);
            return;
        }
    }
    
    if(open(SOURCE_FILE, ">>$file")) {
        print SOURCE_FILE "\n\n$source_line\n\n";
        close(SOURCE_FILE);
        chmod($mode, $file) if($newmode && ($newmode != $mode));
        if(chown($uid, $gid, $file)) {
            return 1;
        } else {
            warn "Changing permissions of $file failed: $!";
            return;
        }
    } else {
        warn "Writing to $file failed: $!";
        chmod($mode, $file) if($newmode && ($newmode != $mode));
        return;
    }
}


sub write_shell_files ($$$$$) {
    my($dir, $shell, $uid, $gid, $subst) = @_;
    
    if(write_env_file("$dir/$shell->{name}", $uid, $gid, $shell->{contents}, $subst)) {
        my $source_line = "$shell->{source} $dir/$shell->{name}";
        
        foreach my $file (@{$shell->{parents}}) {
            return unless(write_source_file("$dir/$file", $uid, $gid, $source_line));
        }
        
        return 1;
    } else {
        # write_env_file will bitch on error
        return;
    }
}

sub write_all_shell_files ($$$$) {
    my($dir, $uid, $gid, $subst) = @_;
    
    while(my($k, $v) = each(%Shells)) {
        return unless(write_shell_files($dir, $v, $uid, $gid, $subst));
    }
    
    return 1;
}
